@@ -122,7 +122,8 @@ class Stage(metaclass=abc.ABCMeta):
         Information about how the NGS result is run
     result_info : dict object
         Key-value pairs storing parsed NGS result
-
+    result_root : Path object
+        Path to stage's result dir
     """
     template_entrances = ['stage.html']
     """Name of templates that will trigger :py:meth:`render`.
@@ -286,7 +287,9 @@ class Stage(metaclass=abc.ABCMeta):
             )
             return self.job_info.root_path
 
-        folder_pattern = r"^\d+_{}$".format(self.result_foldername)
+        folder_pattern = r"^(\d+_|){}$".format(self.result_foldername)
+        logger.debug(
+            "Result foldername regex pattern: {}".format(folder_pattern))
         valid_name = re.compile(folder_pattern).match
 
         stage_result_path = [
@@ -737,19 +740,71 @@ def gen_report(pipe_report_cls, job_dir, out_dir):
     report.generate(job_dir, out_dir)
 
 
-def main():
+def main(argv=None):
     """Store the logics for :command:`ngreport`.
 
-    If one wants to use :command:`ngreport`'s functionality, try calling
+    If one just wants the :command:`ngreport`'s functionality, try call
     :func:`gen_report` not this function.
+
+    Anyway, If **args** is passed, it runs like **ngreport** is called.
+    *args* should be a list that passes to :func:`subprocess.check_call`
+    excluding the first element (program name),
+
+        >>> main(argv=['-p', 'ext_pipe.myReport', 'job_mine', '--color'])
+
+    Notes
+    -----
+    If one wants to emit log messages in their custom modules, NGClouds now
+    recognizes them by default.
+
+    .. code-block:: python3
+
+        # in ext_pipe.py
+        import logging
+        logger = logging.getLogger("external.{}".format(__name__))
+        from ngcloud.report import main as _main
+
+        class MyStage(Stage):
+            def parse(self):
+                logger.warning("My custom warnning")  # will appear in the log
+
+        def main():
+            argv = ["some", "args", "to", "ngreport"]
+            _main(argv=argv)  # If argv not specified, read from sys.argv[1:]
+
+    However, the logging messages will *pollutes* stderr.
+    If that is not desired, use :func:`gen_report`.
+    You can get all NGCloud's log to stderr by
+
+    .. code-block:: python3
+
+        console = logging.StreamHandler()
+        console.setFormatter(logging.Formatter(
+            "[%(levelname)s][%(name)s] %(message)s"))
+        ng_logger = logging.getLogger("ngcloud")
+        ng_logger.addHandler(console)
+        ng_logger.setLevel(logging.INFO)
+
+    .. seealso::
+
+        See module Python standard library :py:mod:`logging` for its flexible
+        logging functionality. You are encouraged to use it.
+
+    .. versionchanged:: 0.3
+
+        Add **argv**; include external loggers.
+
     """
     # setup console logging
     console = logging.StreamHandler()
-    pkg_logger = logging.getLogger("ngcloud")
-    pkg_logger.addHandler(console)
+    all_loggers = logging.getLogger()
+    all_loggers.addHandler(console)
 
     # read arguments from command line
-    args = docopt(_SCRIPT_DOC, version=ng.__version__)
+    if argv:
+        args = docopt(_SCRIPT_DOC, argv=argv, version=ng.__version__)
+    else:
+        args = docopt(_SCRIPT_DOC, version=ng.__version__)
 
     # set logging level
     if args['--verbose'] == 1:
@@ -758,13 +813,18 @@ def main():
         loglevel = logging.DEBUG
     else:
         loglevel = logging.WARNING
-    pkg_logger.setLevel(loglevel)
+    all_loggers.setLevel(loglevel)
 
     # set log format
     log_fmt = '[%(levelname)-7s][%(name)-8s][%(funcName)-8s] %(message)s'
-
     if args['--log-time']:
         log_fmt = '[%(asctime)s]' + log_fmt
+
+    log_formatter = logging.Formatter(
+        log_fmt,
+        '%Y-%m-%d %H:%M:%S'
+    )
+
     color_log_fmt = (
         '%(log_color)s%(levelname)-7s%(reset)s %(cyan)s%(name)-8s%(reset)s '
         '%(log_color)s[%(funcName)s]%(reset)s %(message)s'
@@ -772,11 +832,6 @@ def main():
     if args['--log-time']:
         color_log_fmt = '%(asctime)s ' + color_log_fmt
 
-    # set color log output
-    log_formatter = logging.Formatter(
-        log_fmt,
-        '%Y-%m-%d %H:%M:%S'
-    )
     if args['--color']:
         try:
             import colorlog
